#include "kinectdepthconvert.h"
#include <stdio.h>


//---------------------------------------------------------

KinectDepthConvert::KinectDepthConvert(const char* lutFileName)
{	
	printf( "Loading LUT from file %s.\n", lutFileName);
	loadLUT(lutFileName);
	printf( "LUT size is %d.\n", lutSize());
}

//---------------------------------------------------------

void KinectDepthConvert::loadLUT(const char* lutFileName)
{
	FILE *f;
	
	lutR.clear();
	lutG.clear();
	lutB.clear();
	
	f = fopen( lutFileName, "r");
	if( f == NULL )
		return;
	
	unsigned char r, g, b;	
	while(true)
	{
		int ret = fscanf( f, "%hhu %hhu %hhu", &r, &g, &b);
		
		if( ret == 0  ||  ret == EOF)
		{
			fclose(f);
			return;
		}
		
		lutR.push_back(r);
		lutG.push_back(g);
		lutB.push_back(b);
	}
}

//---------------------------------------------------------

IplImage* KinectDepthConvert::convertToColor( const IplImage *depthImg, bool bBGR)
{
	if( lutSize() == 0 )
	{
		fprintf( stderr, "%s : lutsize is zero!\n", 
			__func__);
		return NULL;
	}	
		
	if( depthImg == NULL )
	{
		fprintf( stderr, "%s : input depth image is zero!.\n", 
			__func__);
		return NULL;
	}
		
	if( depthImg->nChannels != 1 )
	{
		fprintf( stderr, "%s : input depth image must have 1 channel.\n", 
			__func__);
		return NULL;
	}
		
	if( depthImg->depth != IPL_DEPTH_16U )
	{
		fprintf( stderr, "%s : input depth image must be 16 bits unsigned.\n",
		 	__func__);
		return NULL;
	}			
	
	// create color image
	IplImage *clrImg = cvCreateImage( cvGetSize(depthImg), IPL_DEPTH_8U, 3);
	if( clrImg == NULL )
	{
		fprintf( stderr, "Failed to create color image.\n");
		return NULL;
	}
	
	// channels order
	int channelB = 0;
	int channelG = 1;
	int channelR = 2;
	if( ! bBGR )
	{
		channelB = 2;
		channelR = 0;
	}
		
	// convert
	for( int l = 0; l < depthImg->height; l++)
	{
		for( int c = 0; c < depthImg->width; c++)
		{
			// Kinect depth image use 11 bits
			int depth = ((int) cvGet2D( depthImg, l, c).val[0]) / 32;
			int pixId = (l * depthImg->width) + c;
			// this works only with 8 bits / channel image
			clrImg->imageData[pixId*3 + channelR] = lutR[depth];
			clrImg->imageData[pixId*3 + channelG] = lutG[depth];
			clrImg->imageData[pixId*3 + channelB] = lutB[depth];
		}
	}
	
	return clrImg;
}

