
#ifndef KINECTDEPTHCONVERT_H
#define KINECTDEPTHCONVERT_H

#include <cv.h>
#include <vector>

/**
*  Convert Kinect 11/16 bits depth image to 3 channels 8 bits color.
*/
class KinectDepthConvert
{
public:
	/**
		Class constructor.
		
		@param  lutFileName  LUT file name.
	*/
	KinectDepthConvert(const char* lutFileName);

	/**
		Class destructor.
	*/
	~KinectDepthConvert()  { }

	/** 
		Convert 16 bits depth image to color image.
		
		@param  depth  Depth image.
		@param  bBGR  convert to BGR if true, to RGB if false.
		@return  3 channels 8 bits color image. Must be released after use.
	*/
	IplImage* convertToColor( const IplImage *src, bool bBGR = true); 

	/** 
		Get LUT size.
		
		@return  LUT size.
	*/
	int lutSize(void)  { return lutR.size(); }

protected:
	/**
	  Load lookup table from file.
	  
	  @param  lutFileName  LUT File name.
	*/
	void loadLUT(const char* lutFileName);

	//! LUT storage
	std::vector<unsigned char> lutR, lutG, lutB;
};

#endif // KINECTDEPTHCONVERT_H
